Package: inTextSummaryTable
Type: Package
Title: Creation of in-Text Summary Table
Version: 3.4.0
Date: 2026-05-04
Authors@R: c(
  person(given = "Laure", family = "Cougnaud", role = c("aut", "cre"),
  email = "laure.cougnaud@openanalytics.eu"),
  person(given = "Michela", family = "Pasetto", role = "aut",
  email = "michela.pasetto@openanalytics.eu"),
  person(given = "Margaux", family = "Faes", 
  role = "rev", comment = "tests"),
  person(given = "Open Analytics", role = "cph")
  )
Description: Creation of tables of summary statistics or counts for clinical data (for 'TLFs'). 
  These tables can be exported as in-text table (with the 'flextable' package) for a Clinical Study Report 
  (Word format) or a 'topline' presentation (PowerPoint format), 
  or as interactive table (with the 'DT' package) to an html document for clinical data review.
Imports: clinUtils (>= 0.1.0), cowplot, flextable (>= 0.5.5), ggplot2,
        ggrepel, magrittr, methods, officer, plyr, reshape2 (>= 1.4),
        scales, stats, utils
Suggests: htmltools, knitr, rmarkdown, testthat, xml2, tools, tibble,
        extrafont
URL: https://github.com/openanalytics/inTextSummaryTable
BugReports: https://github.com/openanalytics/inTextSummaryTable/issues
License: MIT + file LICENSE
VignetteBuilder: knitr
SystemRequirements: pandoc (to export an interactive summary table to a
        file)
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-04 14:29:24 UTC; root
Author: Laure Cougnaud [aut, cre],
  Michela Pasetto [aut],
  Margaux Faes [rev] (tests),
  Open Analytics [cph]
Maintainer: Laure Cougnaud <laure.cougnaud@openanalytics.eu>
Repository: CRAN
Date/Publication: 2026-05-04 15:50:02 UTC
Built: R 4.6.0; ; 2026-05-04 17:23:57 UTC; unix
