# Generated by roxygen2: do not edit by hand

S3method(combine,summaryTable)
S3method(export,summaryTable)
export(combine)
export(combineVariables)
export(computeSummaryStatistics)
export(computeSummaryStatisticsTable)
export(convertVectToBinary)
export(createFlextableWithHeader)
export(cv)
export(export)
export(exportFlextableToDocx)
export(exportSummaryStatisticsTable)
export(formatPercentage)
export(geomCV)
export(geomMean)
export(geomSD)
export(geomSE)
export(getColorPaletteTable)
export(getDimPage)
export(getListing)
export(getMaxNDecimals)
export(getMaxNDecimalsData)
export(getNDecimals)
export(getNDecimalsData)
export(getNDecimalsRule)
export(getStats)
export(getStatsData)
export(getSummaryStatisticsTable)
export(pageDimPresentation)
export(se)
export(subjectProfileSummaryPlot)
export(subjectProfileSummaryTable)
export(tableColorsPresentation)
export(tableColorsReport)
import(cowplot)
import(flextable)
import(ggplot2)
importFrom(clinUtils,clinColors)
importFrom(clinUtils,clinLinetypes)
importFrom(clinUtils,clinShapes)
importFrom(clinUtils,getClinDT)
importFrom(clinUtils,getColorPalette)
importFrom(clinUtils,getLabelVar)
importFrom(clinUtils,getLinetypePalette)
importFrom(clinUtils,getShapePalette)
importFrom(clinUtils,roundHalfUpTextFormat)
importFrom(flextable,as_b)
importFrom(flextable,as_paragraph)
importFrom(flextable,as_sub)
importFrom(flextable,as_sup)
importFrom(flextable,body_add_flextable)
importFrom(flextable,compose)
importFrom(flextable,set_header_df)
importFrom(ggrepel,geom_text_repel)
importFrom(magrittr,"%>%")
importFrom(methods,formalArgs)
importFrom(officer,body_add_break)
importFrom(officer,body_set_default_section)
importFrom(officer,fp_border)
importFrom(officer,fp_text)
importFrom(officer,page_size)
importFrom(officer,prop_section)
importFrom(plyr,colwise)
importFrom(plyr,daply)
importFrom(plyr,ddply)
importFrom(plyr,dlply)
importFrom(plyr,rbind.fill)
importFrom(reshape2,dcast)
importFrom(reshape2,melt)
importFrom(scales,trans_new)
importFrom(stats,as.formula)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(utils,capture.output)
importFrom(utils,hasName)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,write.table)
