#' Statistics in the in-text table package.
#' 
#' In the in-text package, different set of statistics are available.
#' 
#' The statistics are first computed as numeric
#' ('Base statistics' section below),
#' then formatted to be displayed in the table
#' ('Formatted statistics' section below).
#' @template stats-name-base
#' @template stats-name-formatted
#' @template stats-formatting
#' @name inTextSummaryTable-stats
#' @return No return value, used for the documentation of 
#' the \code{stats} parameter
NULL