#' inTextSummaryTable: creation of in-text summary table
#' 
#' @description Tables of summary statistics or count tables are created. 
#' These tables can be exported as in-text table to a Clinical Study Report 
#' (Word format), a topline presentation (PowerPoint format),
#'  or as interactive table to an html document.
#'
#' @details
#' \itemize{
#' \item{To get started with the package, see:\cr
#' \code{vignette("inTextSummaryTable-01-introduction", package = "inTextSummaryTable")}}
#' \item{To get example code for standard in-text tables created with the package, 
#' see:\cr
#' \code{vignette("inTextSummaryTable-03-standardTables", package = "inTextSummaryTable")}}
#' \item{The main function: \code{\link{getSummaryStatisticsTable}} enables
#' to create ready in-text table.}
#' }
#' @docType package
#' @name inTextSummaryTable
#' @aliases inTextSummaryTable-package
NULL