% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableColors.R
\name{tableColorsReport}
\alias{tableColorsReport}
\title{Colors for tables in a report style}
\usage{
tableColorsReport
}
\description{
Default colors are black text on a white background.
}
