% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{pageDimPresentation}
\alias{pageDimPresentation}
\title{Page dimension for powerpoints}
\usage{
pageDimPresentation
}
\description{
Page dimension for powerpoints
}
